
/*
 * archive files
 *
 * $Header: arc.c 1.2 92/12/07 $
 * $Log:	arc.c,v $
 * Revision 1.2  92/12/07  17:17:07  duplain
 * reformatted source.
 * 
 * Revision 1.1  92/09/29  18:02:05  duplain
 * Initial revision
 * 
 */

#include "spark.h"
#include "error.h"

int
do_arc()
{
	error("archiving not supported in this version");
	return (99);
}
