/*
 * archive function prototypes
 *
 * $Header: arc.h 1.1 92/09/29 $
 * $Log:	arc.h,v $
 * Revision 1.1  92/09/29  18:02:29  duplain
 * Initial revision
 * 
 */

#ifndef __ARC_H
#define __ARC_H

int do_arc(void);

#endif /* __ARC_H */
