/*
 * unarchive function prototypes
 *
 * $Header: unarc.h 1.1 92/09/29 $
 * $Log:	unarc.h,v $
 * Revision 1.1  92/09/29  18:02:43  duplain
 * Initial revision
 * 
 */

#ifndef __UNARC_H
#define __UNARC_H

int do_unarc(void);
int do_unsquash(void);

#endif /* __UNARC_H */
